use Evo::Base -strict;
use Test::More tests => 6;
use Evo ':dsl';

dsl_call {foo => 'bar'}, sub {
  is dsl_depth, 1;
  is_deeply dsl_stash, {foo => 'bar'};
  is(dsl_stash('foo'), 'bar');
  ok dsl_check_stash;
  ok dsl_check_stash('foo');

  dsl_stash(foo2 => 'bar2');
  is dsl_stash('foo2'), 'bar2';

};


done_testing;
