package main;
use Evo::Base -strict;
use Test::More;
use Evo::MDN;

my $mdn  = Evo::MDN->new;
my $from = Evo::Base->new;
my $me   = Evo::Base->new;

my @args;
$mdn->subscribe($me, $from, 1, sub { push @args, @_ });
$mdn->broadcast($from, 'foo');
is_deeply \@args, [$me, 'foo', $from];

done_testing;
