use Evo::Base -strict;
use Test::More tests => 5;
use Evo ':func';

asap {
  pass "asap";
  asap { pass "asap" };
};

my $guard = guard always => sub { pass "guard" };
undef $guard;

eval {
  my $guard = guard
    always => sub { pass "guard" },
    error  => sub { pass "guard error" };
  die;
};

done_testing;
