package main;
use Evo::Base -strict;
use Test::More;
use Evo::MDN;
use Test::Fatal qw(lives_ok exception);
use Hash::Util::FieldHash qw(id);

my ($FROM, $MSG);
my $mdn  = Evo::MDN->new;
my $from = Evo::Base->new;
my $me   = Evo::Base->new;

# not implemented when_message
like exception {
  $mdn->subscribe(Evo::Base->new, $from);
},
  qr/when_message/;

like exception {
  $mdn->subscribe($me, $from, 0, sub { });
  $mdn->subscribe($me, $from, 0, sub { });
}, qr/has been subscribed to/i;

done_testing;

